﻿using System.Collections.Generic;
using System.ComponentModel;
using Dataweb.NShape.Advanced;
using Dataweb.NShape;
using cz.zcu.PreSti.Components;
using System.Drawing;
using System;

namespace cz.zcu.PreSti.Shapes
{
    public class NormalDistributionGeneratorSymbol : GeneratorSymbol
    {
        #region ISymbol Members

        public override BaseComponent initializeComponent(IPresentation p)
        {
            NormalDistributionGeneratorModel model = (NormalDistributionGeneratorModel)this.ModelObject;
            NormalDistributionGenerator generator = new NormalDistributionGenerator();
            generator.Mean = model.Mean;
            generator.Variance = model.Variance;
            generator.CropSize = model.CropSize;
            return generator;
        }

        #endregion

        protected internal NormalDistributionGeneratorSymbol(ShapeType shapeType, Template template)
            : base(shapeType, template)
        {
        }


        protected internal NormalDistributionGeneratorSymbol(ShapeType shapeType, IStyleSet styleSet)
            : base(shapeType, styleSet)
        {
        }

        public override Shape Clone()
        {
            Shape result = new NormalDistributionGeneratorSymbol(Type, (Template)null);
            result.CopyFrom(this);
            return result;
        }

        public override void CopyFrom(Shape source)
        {
            base.CopyFrom(source);
        }

        protected override void DrawIcon(Graphics graphics)
        {
            graphics.DrawImage(icons.normal, new Rectangle(X - Diameter / 2, Y - Diameter / 2, Diameter, Diameter));
        }

        internal static Shape CreateInstance(ShapeType shapeType, Template template)
        {
            return new NormalDistributionGeneratorSymbol(shapeType, template);
        }
    }

    public class NormalDistributionGeneratorModel : ModelObjectBase
    {

        #region Fields

        private double mean = 0;
        private double variance = 1;
        private double cropSize = 0;
        private char securityDomainName = 'A';

        private const int PropertyIdMean = 1;
        private const int PropertyIdVariance = 2;
        private const int PropertyIdCropSize = 3;

        #endregion

        #region properties

        [Category("Presentation")]
        [Description("The expected value of a random number")]
        [PropertyMappingId(PropertyIdMean)]
        public virtual double Mean
        {
            get
            {
                return mean;
            }
            set
            {
                mean = value;
            }
        }

        [Category("Presentation")]
        [Description("The expected (mean) value of the square of the deviation of that variable from its expected value or mean")]
        [PropertyMappingId(PropertyIdVariance)]
        public virtual double Variance
        {
            get
            {
                return variance;
            }
            set
            {
                variance = value;
            }
        }

        [Category("Presentation")]
        [Description("Size of cropping area measured from mean value. Values distanced from mean value for more than CropSize are omitted. 0 means no cropping applied.")]
        [PropertyMappingId(PropertyIdCropSize)]
        public virtual double CropSize
        {
            get
            {
                return cropSize;
            }
            set
            {
                cropSize = value;
            }
        }

        #endregion

        public static NormalDistributionGeneratorModel CreateInstance(ModelObjectType modelObjectType) {
			return new NormalDistributionGeneratorModel(modelObjectType);
		}


		protected internal NormalDistributionGeneratorModel(ModelObjectType modelObjectType)
			: base(modelObjectType)
        {
            this.terminalCount = 1;
		}


        protected internal NormalDistributionGeneratorModel(NormalDistributionGeneratorModel source)
			: base(source) {
            Mean = source.Mean;
            Variance = source.Variance;
            CropSize = source.CropSize;
            this.terminalCount = 1;
		}


		public override IModelObject Clone() {
            return new NormalDistributionGeneratorModel(this);
		}


		public override IEnumerable<MenuItemDef> GetMenuItemDefs() {
			throw new NotImplementedException();
		}


		public override void Connect(TerminalId ownTerminalId, IModelObject targetConnector, TerminalId targetTerminalId) {
			throw new NotImplementedException();
		}


		public override void Disconnect(TerminalId ownTerminalId, IModelObject targetConnector, TerminalId targetTerminalId) {
			throw new NotImplementedException();
		}

        public override char SecurityDomainName
        {
            get { return securityDomainName; }
            set
            {
                if (value < 'A' || value > 'Z')
                    throw new ArgumentOutOfRangeException("SecurityDomainName", "The domain qualifier has to be an upper case  ANSI letter (A-Z).");
                securityDomainName = value;
            }
        }


        #region IEntity Members

        public static new IEnumerable<EntityPropertyDefinition> GetPropertyDefinitions(int version)
        {
            foreach (EntityPropertyDefinition pi in ModelObjectBase.GetPropertyDefinitions(version))
                yield return pi;
            yield return new EntityFieldDefinition("Mean", typeof(double));
            yield return new EntityFieldDefinition("Variance", typeof(double));
            yield return new EntityFieldDefinition("CropSize", typeof(double));
        }

        protected override void LoadFieldsCore(IRepositoryReader reader, int version)
        {
            base.LoadFieldsCore(reader, version);
            Mean = reader.ReadDouble();
            Variance = reader.ReadDouble();
            CropSize = reader.ReadDouble();
        }

        protected override void SaveFieldsCore(IRepositoryWriter writer, int version)
        {
            base.SaveFieldsCore(writer, version);
            writer.WriteDouble(Mean);
            writer.WriteDouble(Variance);
            writer.WriteDouble(CropSize);
        }

        #endregion


    }
}
