﻿using System.Collections.Generic;
using System.Drawing;
using Dataweb.NShape.Advanced;
using Dataweb.NShape;
using System.ComponentModel;
using cz.zcu.PreSti.Components;
using System;

namespace cz.zcu.PreSti.Shapes
{
    public class SubdiagramSymbol : RectangleBase, ISymbol
    {
        protected override void InitializeToDefault(IStyleSet styleSet)
        {
            base.InitializeToDefault(styleSet);
            Height = 69;
            Width = 92;
            Text = "Diagram";
        }

        protected override int ControlPointCount
        {
            get { return ActionsCount + EventsCount + SettersCount + GettersCount; }
        }


        public override bool HasControlPointCapability(ControlPointId controlPointId, ControlPointCapabilities controlPointCapability)
        {
            if (controlPointId > 0 && controlPointId < ActionsCount + EventsCount + SettersCount + GettersCount + 1)
                return ((controlPointCapability & ControlPointCapabilities.Connect) != 0);
            else return base.HasControlPointCapability(controlPointId, controlPointCapability);
        }

        public override void CopyFrom(Shape source)
        {
            base.CopyFrom(source);
        }

        public override Shape Clone()
        {
            Shape result = new SubdiagramSymbol(Type, (Template)null);
            result.CopyFrom(this);
            return result;
        }

        protected internal SubdiagramSymbol(ShapeType shapeType, Template template)
            : base(shapeType, template)
        {
        }

        protected override void CalcControlPoints()
        {

            int left = (int)Math.Round(-Width / 2f);
            int top = (int)Math.Round(-Height / 2f);
            int right = left + Width;
            int bottom = top + Height;

            ControlPoints[0].X = left;
            ControlPoints[0].Y = top + Height / 2;
            ControlPoints[1].X = right;
            ControlPoints[1].Y = top + Height / 2;

            for (int i = 0; i < SettersCount; i++)
            {
                int w = Width / SettersCount;
                ControlPoints[i + ActionsCount + EventsCount].X = left + i * w + w / 2;
                ControlPoints[i + ActionsCount + EventsCount].Y = top;
            }

            for (int i = 0; i < GettersCount; i++)
            {
                int w = Width / GettersCount;
                ControlPoints[i + ActionsCount + EventsCount + SettersCount].X = left + i * w + w / 2;
                ControlPoints[i + ActionsCount + EventsCount + SettersCount].Y = bottom;
            }
        }


        protected override void CalcCaptionBounds(int index, out Rectangle captionBounds)
        {
            if (index != 0) throw new IndexOutOfRangeException();
            int left = (int)Math.Round(-Width / 2f);
            int top = (int)Math.Round(-Height / 2f);
            captionBounds = Rectangle.Empty;
            captionBounds.X = left;
            captionBounds.Y = top;
            captionBounds.Width = Width;
            captionBounds.Height = Height;
        }


        protected override bool CalculatePath()
        {
            if (base.CalculatePath())
            {
                int left = (int)Math.Round(-Width / 2f);
                int top = (int)Math.Round(-Height / 2f);
                int right = left + Width;
                int bottom = top + Height;

                Rectangle shapeRect = new Rectangle(left, top, Width, Height);

                Path.Reset();
                Path.StartFigure();
                Path.AddRectangle(shapeRect);
                PointF[] a = { new PointF((float)right, top + Height / 2f - connectorWidth), new PointF((float)right - connectorWidth, top + Height / 2f), new PointF(right, top + Height / 2f + connectorWidth) };
                PointF[] b = { new PointF(left, top + Height / 2f - connectorWidth), new PointF(left + connectorWidth, top + Height / 2f), new PointF(left, top + Height / 2f + connectorWidth) };
                Path.StartFigure();
                Path.FillMode = System.Drawing.Drawing2D.FillMode.Winding;
                Path.AddPolygon(a);
                Path.StartFigure();
                Path.AddPolygon(b);

                Path.StartFigure();
                for (int i = 0; i < SettersCount; i++)
                {
                    int w = Width / SettersCount;
                    int posX = left + i * w + w / 2;
                    Path.AddCurve(new PointF[] { new PointF(posX - 4, top), new PointF(posX, top + 4), new PointF(posX + 4, top) }, 1);
                }

                Path.StartFigure();
                for (int i = 0; i < GettersCount; i++)
                {
                    int w = Width / GettersCount;
                    int posX = left + i * w + w / 2;
                    Path.AddCurve(new PointF[] { new PointF(posX - 4, bottom), new PointF(posX, bottom - 4), new PointF(posX + 4, bottom) }, 1);
                }
                Path.CloseFigure();
                return true;
            }
            return false;
        }

        #region Fields
        private int connectorWidth = 5;
        #endregion

        #region ISymbol Members

        public BaseComponent initializeComponent(IPresentation p)
        {
            SubdiagramModel model = (SubdiagramModel)this.ModelObject;
            Subdiagram subdiagram = new Subdiagram(p, model.SubdiagramName);
            return subdiagram;
        }

        public virtual int ActionsCount
        {
            get { return 1; }
        }

        public virtual int EventsCount
        {
            get { return 1; }
        }

        public virtual int SettersCount
        {
            get { return 7; }
        }

        public virtual int GettersCount
        {
            get { return 7; }
        }

        public virtual string GetTitleForAction(int id)
        {
            switch (id)
            {
                case 0:
                    return "Start the group of stimuli";
            }
            return null;
        }

        public virtual string GetTitleForEvent(int id)
        {
            switch (id)
            {
                case 0:
                    return "End of the group of stimuli";
            }
            return null;
        }

        public virtual string GetTitleForSetter(int id)
        {
            return string.Format("Setter #{0}", id + 1);
        }

        public virtual string GetTitleForGetter(int id)
        {
            return string.Format("Getter #{0}", id + 1); ;
        }

        #endregion

        internal static Shape CreateInstance(ShapeType shapeType, Template template)
        {
            return new SubdiagramSymbol(shapeType, template);
        }

    }

    public class SubdiagramModel : ModelObjectBase
    {

        #region Fields

        private string subdiagramName = "";
        private char securityDomainName = 'A';

        private const int PropertyIdSubdiagramName = 1;

        #endregion

        #region properties

        [Category("Presentation")]
        [Description("Name of assigned subdiagram.")]
        [PropertyMappingId(PropertyIdSubdiagramName)]
        public virtual string SubdiagramName
        {
            get
            {
                return subdiagramName;
            }
            set
            {
                subdiagramName = value;
            }
        }

        #endregion

        public static SubdiagramModel CreateInstance(ModelObjectType modelObjectType)
        {
            return new SubdiagramModel(modelObjectType);
        }


        protected internal SubdiagramModel(ModelObjectType modelObjectType)
            : base(modelObjectType)
        {
            this.terminalCount = 16;
        }


        protected internal SubdiagramModel(SubdiagramModel source)
            : base(source)
        {
            SubdiagramName = source.SubdiagramName;
            this.terminalCount = 16;
        }


        public override IModelObject Clone()
        {
            return new SubdiagramModel(this);
        }


        public override IEnumerable<MenuItemDef> GetMenuItemDefs()
        {
            throw new NotImplementedException();
        }

        public override void Connect(TerminalId ownTerminalId, IModelObject targetConnector, TerminalId targetTerminalId)
        {
        }


        public override void Disconnect(TerminalId ownTerminalId, IModelObject targetConnector, TerminalId targetTerminalId)
        {
        }

        public override char SecurityDomainName
        {
            get { return securityDomainName; }
            set
            {
                if (value < 'A' || value > 'Z')
                    throw new ArgumentOutOfRangeException("SecurityDomainName", "The domain qualifier has to be an upper case  ANSI letter (A-Z).");
                securityDomainName = value;
            }
        }


        #region IEntity Members

        public static new IEnumerable<EntityPropertyDefinition> GetPropertyDefinitions(int version)
        {
            foreach (EntityPropertyDefinition pi in ModelObjectBase.GetPropertyDefinitions(version))
                yield return pi;
            yield return new EntityFieldDefinition("SubdiagramName", typeof(string));
        }

        protected override void LoadFieldsCore(IRepositoryReader reader, int version)
        {
            base.LoadFieldsCore(reader, version);
            SubdiagramName = reader.ReadString();
        }

        protected override void SaveFieldsCore(IRepositoryWriter writer, int version)
        {
            base.SaveFieldsCore(writer, version);
            writer.WriteString(SubdiagramName);
        }

        #endregion
    }


}