﻿using System;
using System.Drawing;
using Dataweb.NShape.Advanced;
using Dataweb.NShape;

namespace cz.zcu.PreSti.Shapes
{
    public class TrialSymbol : RectangleBase
    {
        protected override void InitializeToDefault(IStyleSet styleSet)
        {
            base.InitializeToDefault(styleSet);
            Height = 60;
            Width = 80 + connectorWidth;
        }

        public override Shape Clone()
        {
            Shape result = new TrialSymbol(Type, (Template)null);
            result.CopyFrom(this);
            return result;
        }


        protected override int ControlPointCount
        {
            get { return 3; }
        }


        public override bool HasControlPointCapability(ControlPointId controlPointId, ControlPointCapabilities controlPointCapability)
        {
            switch (controlPointId)
            {
                case 1:
                case 2:
                case 3:
                    return ((controlPointCapability & ControlPointCapabilities.Connect) != 0);
                default:
                    return base.HasControlPointCapability(controlPointId, controlPointCapability);
            }
        }


        protected internal TrialSymbol(ShapeType shapeType, Template template)
            : base(shapeType, template)
        {
        }


        protected override void CalcControlPoints()
        {

            int left = (int)Math.Round(-Width / 2f);
            int top = (int)Math.Round(-Height / 2f);
            int right = left + Width;
            int bottom = top + Height;

            ControlPoints[0].X = left;
            ControlPoints[0].Y = top + Height / 2;
            ControlPoints[1].X = right;
            ControlPoints[1].Y = top + Height / 2;
            ControlPoints[2].X = left + Width / 2;
            ControlPoints[2].Y = bottom;
        }


        protected override void CalcCaptionBounds(int index, out Rectangle captionBounds)
        {
            if (index != 0) throw new IndexOutOfRangeException();
            int left = (int)Math.Round(-Width / 2f);
            int top = (int)Math.Round(-Height / 2f);
            captionBounds = Rectangle.Empty;
            captionBounds.X = left;
            captionBounds.Y = top;
            captionBounds.Width = Width;
            captionBounds.Height = Height;
        }


        protected override bool CalculatePath()
        {
            if (base.CalculatePath())
            {
                int left = (int)Math.Round(-Width / 2f);
                int top = (int)Math.Round(-Height / 2f);
                int right = left + Width;
                int bottom = top + Height;

                shapeRect.X = left;
                shapeRect.Y = top;
                shapeRect.Width = Width - connectorWidth;
                shapeRect.Height = Height;

                RectangleF shapeRect2 = RectangleF.Empty;
                shapeRect2.X = left + Width / 2f - terminatorWidth / 2;
                shapeRect2.Y = bottom - terminatorWidth;
                shapeRect2.Width = terminatorWidth;
                shapeRect2.Height = terminatorWidth;

                Path.Reset();
                Path.StartFigure();
                Path.AddRectangle(shapeRect);
                PointF[] a = {new PointF((float)right - connectorWidth, top + Height / 2f - connectorWidth), new PointF(right, top + Height / 2f), new PointF((float)right - connectorWidth, top + Height / 2f + connectorWidth)};
                PointF[] b = { new PointF(left, top + Height / 2f - connectorWidth), new PointF(left + connectorWidth, top + Height / 2f), new PointF(left, top + Height / 2f + connectorWidth) };
                Path.AddPolygon(a);
                Path.AddPolygon(b);
                Path.AddRectangle(shapeRect2);
                Path.AddLine(left + Width / 2f - terminatorWidth / 2, bottom - terminatorWidth, left + Width / 2f + terminatorWidth / 2, bottom);
                Path.AddLine(left + Width / 2f + terminatorWidth / 2, bottom - terminatorWidth, left + Width / 2f - terminatorWidth / 2, bottom);
                Path.CloseFigure();
                return true;
            }
            return false;
        }


        #region Fields
        private int connectorWidth = 5;
        private int terminatorWidth = 10;
        private Rectangle shapeRect;
        #endregion

        internal static Shape CreateInstance(ShapeType shapeType, Template template)
        {
            return new TrialSymbol(shapeType, template);
        }
    }

}
