using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.Threading;
using System.Diagnostics;

namespace cz.zcu.PreSti.XNADirectX
{
    public class XNADirectXPresentation : Microsoft.Xna.Framework.Game, IPresentation
    {
        // Audio objects
        private AudioEngine engine;
        private SoundBank soundBank;
        private WaveBank waveBank;

        private DateTime startTime;
        private bool isLoading = true;
        private int loadedSymbols = 0;
        private string lastError = null;
        private Stopwatch stopwatch = new Stopwatch();

        GraphicsDevice device;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        ISymbols symbols;
        private object monitor = 1;

        private SortedSet<RunnableComponent> Layers = new SortedSet<RunnableComponent>();
        private List<RunnableComponent> EntryPoints = new List<RunnableComponent>();

        public List<RunnableComponent> entryPoints
        {
            get
            {
                return EntryPoints;
            }
            set
            {
                EntryPoints = value;
            }
        }

        public SortedSet<RunnableComponent> layers
        {
            get
            {
                return Layers;
            }
        }

        public DateTime StartTime
        {
            get
            {
                return startTime;
            }
        }

        public bool IsLoading
        {
            get
            {
                return isLoading;
            }
        }

        public int LoadedSymbols
        {
            get
            {
                return loadedSymbols;
            }
        }

        public int SymbolsCount
        {
            get
            {
                return symbols.GetSymbolsCount();
            }
        }

        public string LastError
        {
            get
            {
                string error = this.lastError;
                this.lastError = null;
                return error;
            }
            set
            {
                this.lastError = value;
            }
        }

        public bool HasError
        {
            get
            {
                return this.lastError != null;
            }
        }


        public XNADirectXPresentation(ISymbols P_symbols)
        {
            base.Initialize();
            graphics = new GraphicsDeviceManager(this);
            symbols = P_symbols;
        }

        public void initDirectX(Control window, int width, int height, bool windowed)
        {
            Console.WriteLine("Init");
            graphics.PreferredBackBufferWidth = width;
            graphics.PreferredBackBufferHeight = height;
            graphics.IsFullScreen = (!(windowed));
            graphics.ApplyChanges();

        }


        protected override void Draw(GameTime gameTime)
        {
            Console.WriteLine(gameTime.ToString());
            draw();
        }

        public void draw()
        {
            /*Console.WriteLine("Draw");
            Monitor.Enter(monitor);
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            try
            {
                device.Clear(ClearOptions.Target, Color.Red, 1.0f, 0);
                spriteBatch.Begin(SpriteSortMode.BackToFront, BlendState.AlphaBlend);
                RunnableComponent[] layers = new RunnableComponent[this.layers.Count()];
                this.layers.CopyTo(layers, 0, layers.Count());
                foreach (RunnableComponent l in layers)
                {
                    l.draw();
                }
                spriteBatch.End();
            }
               
            catch (NullReferenceException)
            {
                return;
            }
            finally
            {
                Monitor.Exit(monitor);
            }*/
        }

        public void initSound()
        {
            /*engine = new AudioEngine("Content\\test.xgs");
            soundBank = new SoundBank(engine, "Content\\Sound Bank.xsb");
            waveBank = new WaveBank(engine, "Content\\Wave Bank.xwb");

            Cue cue = soundBank.GetCue("kaboom");
            cue.Play();

            // wait till it ends
            while (cue.IsPlaying)
            {
                Console.Out.WriteLine("playing");
            }*/
        }

        public void playSound(String Ps_soundName)
        {
            engine = new AudioEngine("Content\\SoundContainer.xgs");
            soundBank = new SoundBank(engine, "Content\\SoundBank.xsb");
            waveBank = new WaveBank(engine, "Content\\WaveBank.xwb");

            Cue cue = soundBank.GetCue(Ps_soundName);
            cue.Play();

            // wait till it ends
            while (cue.IsPlaying)
            {
                Thread.Sleep(10);
                //Console.Out.WriteLine("playing sound " + Ps_soundName);
            }
        }

        public void initComponents()
        {
            isLoading = true;
            entryPoints = new List<RunnableComponent>();
            Dictionary<ISymbol, BaseComponent> dict = new Dictionary<ISymbol, BaseComponent>();
            foreach (ISymbol s in symbols.GetEntryPoints())
            {
                loadedSymbols++;
                entryPoints.Add((RunnableComponent)(s.initializeComponent(this)));
                dict.Add(s, entryPoints.Last());
                observeChildsR(dict, s);
            }
            isLoading = false;
        }

        private void observeChildsR(Dictionary<ISymbol, BaseComponent> dict, ISymbol s)
        {
            BaseComponent curComp = dict[s];
            List<ITrigger> triggers = symbols.GetTriggers(s);
            foreach (ITrigger t in triggers)
            {
                loadedSymbols++;
                if (t.IsDescendantConnected())
                {
                    ISymbol otherS = t.GetDescendant();
                    RunnableComponent otherComp;
                    if (dict.ContainsKey(otherS))
                    {
                        otherComp = (RunnableComponent)dict[otherS];
                    }
                    else
                    {
                        otherComp = (RunnableComponent)(otherS.initializeComponent(this));
                        otherComp.Redraw += new EventHandler(onRedraw);
                        loadedSymbols++;
                    }
                    otherComp.update();
                    ((RunnableComponent)curComp).setTrigOut(t.GetEventN(), otherComp, t.GetConnectedShapeActionN(), t.GetDelay());
                    if (!dict.ContainsKey(otherS))
                    {
                        dict.Add(otherS, otherComp);
                        observeChildsR(dict, otherS);
                    }
                }
            }
            List<IConnection> connections = symbols.GetSetterConnections(s);
            foreach (IConnection c in connections)
            {
                loadedSymbols++;
                if (c.IsFullyConnected() && curComp is RunnableComponent) //Odstranit druhou st podmnky
                {
                    ISymbol otherS = c.GetConnectedShape();
                    BaseComponent otherComp;
                    if (dict.ContainsKey(otherS))
                    {
                        otherComp = dict[otherS];
                    }
                    else
                    {
                        otherComp = (BaseComponent)(otherS.initializeComponent(this));
                        loadedSymbols++;
                    }
                    curComp.setupSetter(c.GetSetterN(), otherComp, c.GetConnectedShapeGetterN());
                    if (!dict.ContainsKey(otherS))
                    {
                        dict.Add(otherS, otherComp);
                        observeChildsR(dict, otherS);
                    }
                }
            }
        }


        public void run()
        {
            startTime = DateTime.Now;
            foreach (RunnableComponent entryPoint in entryPoints)
                entryPoint.trigIn(0);
        }

        public event EventHandler Terminated;
        public event EventHandler Redraw;

        private void onRedraw(object sender, EventArgs e)
        {
            if (Redraw != null) Redraw(sender, e);
        }

        public void terminate()
        {
            foreach (RunnableComponent c in this.entryPoints)
            {
                this.terminate(c);
            }

            if (this.Terminated != null)
            {
                this.Terminated(this, EventArgs.Empty);
            }
        }

        public void terminate(RunnableComponent c)
        {
            foreach (RunnableComponent.Trigger t in c.TrigOuts)
            {
                t.terminate();
                t.Component.terminate();
                terminate(t.Component);
            }
        }

        public bool supportsGraphics()
        {
            return true;
        }

        public bool supportsAudio()
        {
            return true;
        }

        public void startClock()
        {
            Console.Out.WriteLine("Starting clock");
            stopwatch.Reset();
            stopwatch.Start();
        }

        public void stopClock()
        {
            stopwatch.Stop();
            Console.Out.WriteLine("Stopping clock - time(in ms) " + stopwatch.ElapsedMilliseconds);
            Console.Out.WriteLine("Stopping clock - time(ticks - 10000 is one ms) " + stopwatch.ElapsedTicks);
        }
    }
}
